#!/usr/bin/perl
#
# script to get temperature from hddtemp and dump into rrd
#
# $Id: hddrrd.pl 284 2007-10-18 13:28:08Z weizen_42 $
#
# created 23 February 2006 by 'weizen_42'
#
# 2006-02-xx weizen_42        several modifications
# 2006-03-12 weizen_42        UTF-8 hacks from makegraphs
# 2006-03-28 weizen_42        does not break when HD not in database
# 2006-05-26 weizen_42        can also use smartctl
# 2006-11-17 weizen_42        additional language hack(s)
# 2007-10-14 weizen_42        additional language hack(s)
#


use strict;
use RRDs;
require "/var/ipcop/general-functions.pl";
require "${General::swroot}/lang.pl";

#device name
my $hdd_device = "/dev/harddisk";
# path for rrd database
my $path_rrd = "/var/log/rrd";
# path for graphs
my $path_graphs = "/home/httpd/html/graphs";

# path to binary helpers
my $path_hddtemp  = "/usr/sbin/hddtemp";
my $path_smartctl = "/usr/local/sbin/smartctl";

my %hddgraphsettings=();
$hddgraphsettings{"SMARTCTL_OPTIONS"} = "";
if ( -e "/var/ipcop/hddgraph/settings" ) 
{
  &General::readhash("/var/ipcop/hddgraph/settings", \%hddgraphsettings);
}

my $temp;
my $ERROR;

# Added for conversion of utf-8 characters
use Encode 'from_to';
my %tr=();

# From makegraphs:
# Force language back to English (ugly hack!)
# Modified to only force if we are unable to convert charset 
# from utf-8
if (${Lang::language} =~ /^(ar|bg|el|fa|gu|ja|ru|th|ur|vi|zh|zt)$/) 
{
  eval `/bin/cat "${General::swroot}/langs/en.pl"`;

  # add our AddOn texts as well
  $tr{'harddisk temperature'} = 'Harddisk temperature';
  $tr{'hdd temperature in'} = 'Harddisk temperature in';
}
else 
{
  %tr=%Lang::tr;    # use translated version for other languages
}


sub updatehdddata 
{
  if ( ! -e "$path_rrd/hddtemp.rrd") 
  {
    # database did not exist -> create
    RRDs::create ("$path_rrd/hddtemp.rrd", "--step=300",
      "DS:temperature:GAUGE:600:0:100",
      "RRA:AVERAGE:0.5:1:576",
      "RRA:AVERAGE:0.5:6:672",
      "RRA:AVERAGE:0.5:24:732",
      "RRA:AVERAGE:0.5:144:1460");
    $ERROR = RRDs::error;
    print "Error in RRD::create for hdd: $ERROR\n" if $ERROR;
  }

  $temp = 0;
  my $hdd_output = '';
  my $smart_output = '';

  if ( -e "$path_smartctl" )
  {
    system("$path_smartctl -iHA $hddgraphsettings{'SMARTCTL_OPTIONS'} $hdd_device > /var/log/hddgraph_smartctl_out");
  }

  if ( -e "$path_hddtemp" )
  {
    $hdd_output = `$path_hddtemp -qn $hdd_device`;

    # I know 4 response possible responses:
    #
    # /dev/harddisk: harddisk type: S.M.A.R.T. not available
    # /dev/harddisk: harddisk type: no sensor
    # /dev/harddisk: harddisk type: 37C or F
    # 37

    if ( index($hdd_output, "S.M.A.R.T.") != -1 )
    {
      $temp = 0;
    }
    elsif ( index($hdd_output, "no sensor") != -1 )
    {
      $temp = 1;
    }
    elsif ( index($hdd_output, "$hdd_device") != -1 )
    {
      $hdd_output =~ /.*:.*:\s*(\d+).*\s/;
      $temp = $1;
    }
    else
    {
      $hdd_output =~ /(\d+)\s/;
      $temp = $1;
    }
  }
  elsif ( -e "/var/log/hddgraph_smartctl_out" )
  {
    $hdd_output = `cat /var/log/hddgraph_smartctl_out | grep Temperature_`;
    my @t = split(/\s+/,$hdd_output);
    $temp = $t[9];
  }
  else
  {
    $temp = 0;
  }

#  print "hdd  $hdd_output \n";
#  print "temp $temp \n";

  RRDs::update ("$path_rrd/hddtemp.rrd", "-t", "temperature", "N:$temp");

  $ERROR = RRDs::error;
  print "Error in RRD::update for hdd: $ERROR\n" if $ERROR;
}


sub updatehddgraph {
  my $period    = $_[0];

  RRDs::graph ("$path_graphs/hddtemp-$period.png",
    "--start", "-1$period", "-aPNG", "-i", "-z",
    "--alt-y-grid", "-w 600", "-h 100",
    "--color", "SHADEA#EAE9EE",
    "--color", "SHADEB#EAE9EE",
    "--color", "BACK#EAE9EE",
    "-t $tr{'harddisk temperature'} ($tr{'graph per'} $tr{$period})",
    "DEF:temperature=$path_rrd/hddtemp.rrd:temperature:AVERAGE",
    "LINE2:temperature#0000FF:$tr{'hdd temperature in'} C",
    "GPRINT:temperature:MAX:$tr{'maximal'}\\:%3.0lf C",
    "GPRINT:temperature:AVERAGE:$tr{'average'}\\:%3.0lf C",
    "GPRINT:temperature:LAST:$tr{'current'}\\:%3.0lf C",
  );
  $ERROR = RRDs::error;
  print "Error in RRD::graph for cpu: $ERROR\n" if $ERROR;

}

# From makegraphs:
###
### utf8 conversion
###
if ((${Lang::language} eq 'cs') || (${Lang::language} eq 'hu') || (${Lang::language} eq 'pl') || (${Lang::language} eq 'sk')) 
{
  # Czech, Hungarian, Polish and Slovak character set
  foreach my $key(keys %Lang::tr) 
  {
    from_to($tr{$key}, "utf-8", "iso-8859-2");
  }
}
elsif (${Lang::language} eq 'lt') 
{
  # Lithuanian
  foreach my $key(keys %Lang::tr) 
  {
    from_to($tr{$key}, "utf-8", "iso-8859-4");
  }
} 
elsif (${Lang::language} eq 'tr') 
{
  # Turkish
  foreach my $key(keys %Lang::tr) 
  {
    from_to($tr{$key}, "utf-8", "iso-8859-9");
  }
} 
else 
{
  foreach my $key(keys %Lang::tr) 
  {
    from_to($tr{$key}, "utf-8", "iso-8859-1"); 
  }
}


updatehdddata();
updatehddgraph ("day");
updatehddgraph ("week");
updatehddgraph ("month");
updatehddgraph ("year");
